import { Account, Connection, PublicKey, Transaction, TransactionSignature } from '@solana/web3.js';
import { WideBits } from './layout';
import BN from 'bn.js';
import { Market, OpenOrders, Orderbook } from '@project-serum/serum';
import { Order } from '@project-serum/serum/lib/market';
import Wallet from '@project-serum/sol-wallet-adapter';
export declare class MangoGroup {
    publicKey: PublicKey;
    accountFlags: WideBits;
    tokens: PublicKey[];
    vaults: PublicKey[];
    indexes: {
        lastUpdate: BN;
        borrow: number;
        deposit: number;
    };
    spotMarkets: PublicKey[];
    oracles: PublicKey[];
    signerNonce: BN;
    signerKey: PublicKey;
    dexProgramId: PublicKey;
    totalDeposits: number[];
    totalBorrows: number[];
    maintCollRatio: number;
    initCollRatio: number;
    srmVault: PublicKey;
    admin: PublicKey;
    borrowLimits: number[];
    mintDecimals: number[];
    oracleDecimals: number[];
    nativeSrm: number | null;
    constructor(publicKey: PublicKey, decoded: any, nativeSrm?: number);
    getPrices(connection: Connection): Promise<number[]>;
    getMarketIndex(spotMarket: Market): number;
    getTokenIndex(token: PublicKey): number;
    getBorrowRate(tokenIndex: number): number;
    getDepositRate(tokenIndex: number): number;
    getUiTotalDeposit(tokenIndex: number): number;
    getUiTotalBorrow(tokenIndex: number): number;
}
export declare class MarginAccount {
    publicKey: PublicKey;
    createTime: number;
    accountFlags: WideBits;
    mangoGroup: PublicKey;
    owner: PublicKey;
    deposits: number[];
    borrows: number[];
    openOrders: PublicKey[];
    srmBalance: number;
    openOrdersAccounts: (OpenOrders | undefined)[];
    constructor(publicKey: PublicKey, decoded: any);
    getNativeDeposit(mangoGroup: MangoGroup, tokenIndex: number): number;
    getNativeBorrow(mangoGroup: MangoGroup, tokenIndex: number): number;
    getUiDeposit(mangoGroup: MangoGroup, tokenIndex: number): number;
    getUiBorrow(mangoGroup: MangoGroup, tokenIndex: number): number;
    getUiSrmBalance(): number;
    loadOpenOrders(connection: Connection, dexProgramId: PublicKey): Promise<(OpenOrders | undefined)[]>;
    toPrettyString(mangoGroup: MangoGroup, prices: number[]): string;
    computeValue(mangoGroup: MangoGroup, prices: number[]): number;
    getValue(connection: Connection, mangoGroup: MangoGroup): Promise<number>;
    getDeposits(mangoGroup: MangoGroup): number[];
    getAssets(mangoGroup: MangoGroup): number[];
    getLiabs(mangoGroup: MangoGroup): number[];
    getAssetsVal(mangoGroup: MangoGroup, prices: number[]): number;
    getLiabsVal(mangoGroup: MangoGroup, prices: number[]): number;
    getCollateralRatio(mangoGroup: MangoGroup, prices: number[]): number;
    cancelAllOrdersByMarket(connection: Connection, client: MangoClient, programId: PublicKey, mangoGroup: MangoGroup, market: Market, bids: Orderbook, asks: Orderbook, owner: Account): Promise<TransactionSignature[]>;
}
export declare class MangoClient {
    sendTransaction2(connection: Connection, transaction: Transaction, payer: Account, additionalSigners: Account[]): Promise<TransactionSignature>;
    sendTransaction(connection: Connection, transaction: Transaction, payer: Account, additionalSigners: Account[]): Promise<TransactionSignature>;
    initMangoGroup(connection: Connection, programId: PublicKey, payer: PublicKey): Promise<void>;
    initMarginAccount(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account): Promise<PublicKey>;
    deposit(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, tokenAcc: PublicKey, quantity: number): Promise<TransactionSignature>;
    withdraw(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, tokenAcc: PublicKey, quantity: number): Promise<TransactionSignature>;
    borrow(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, quantity: number): Promise<TransactionSignature>;
    settleBorrow(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, quantity: number): Promise<TransactionSignature>;
    /**
     * Call SettleFunds on each market, then call SettleBorrow for each token in one transaction
     * @param connection
     * @param programId
     * @param mangoGroup
     * @param marginAccount
     * @param markets
     * @param owner
     */
    settleAll(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, markets: Market[], owner: Account): Promise<TransactionSignature | null>;
    liquidate(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, liqeeMarginAccount: MarginAccount, // liquidatee marginAccount
    liqor: Account, // liquidator
    tokenAccs: PublicKey[], depositQuantities: number[]): Promise<TransactionSignature>;
    depositSrm(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, srmAccount: PublicKey, quantity: number): Promise<TransactionSignature>;
    withdrawSrm(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, srmAccount: PublicKey, quantity: number): Promise<TransactionSignature>;
    placeOrder(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, spotMarket: Market, owner: Account, side: 'buy' | 'sell', price: number, size: number, orderType?: 'limit' | 'ioc' | 'postOnly', clientId?: BN): Promise<TransactionSignature>;
    settleFunds(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, spotMarket: Market): Promise<TransactionSignature>;
    cancelOrder(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, spotMarket: Market, order: Order): Promise<TransactionSignature>;
    getMangoGroup(connection: Connection, mangoGroupPk: PublicKey, srmVaultPk?: PublicKey): Promise<MangoGroup>;
    getMarginAccount(connection: Connection, marginAccountPk: PublicKey, dexProgramId: PublicKey): Promise<MarginAccount>;
    getAllMarginAccounts(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup): Promise<MarginAccount[]>;
    getMarginAccountsForOwner(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account | Wallet): Promise<MarginAccount[]>;
}
//# sourceMappingURL=client.d.ts.map