"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeSettleBorrowInstruction = exports.makeSettleFundsInstruction = exports.makeCancelOrderInstruction = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const web3_js_1 = require("@solana/web3.js");
const layout_1 = require("./layout");
const token_instructions_1 = require("@project-serum/serum/lib/token-instructions");
// export function makeInitMangoGroupInstruction(
//   programId: PublicKey,
//   mangoGroupPk: PublicKey,
//   signerKey: PublicKey,
//   dexProgramId: PublicKey,
//   srmVaultPk: PublicKey,
//   mintPks: PublicKey[],
//   vaultPks: PublicKey[],
//   spotMarketPks: PublicKey[],
//   oraclePks: PublicKey[],
//   signerNonce: BN,
//   maintCollRatio: number,
//   initCollRatio: number
// ): TransactionInstruction {
//
//   new BN()
//   const keys = [
//     { isSigner: false, isWritable: true, pubkey: mangoGroupPk},
//     { isSigner: true, isWritable: false,  pubkey: ownerPk },
//     { isSigner: false,  isWritable: true, pubkey: marginAccountPk },
//     { isSigner: false, isWritable: false, pubkey: SYSVAR_CLOCK_PUBKEY },
//     { isSigner: false, isWritable: false, pubkey: dexProgramId },
//     { isSigner: false, isWritable: true, pubkey: spotMarketPk },
//     { isSigner: false, isWritable: true, pubkey: bidsPk },
//     { isSigner: false, isWritable: true, pubkey: asksPk },
//     { isSigner: false, isWritable: true, pubkey: openOrdersPk },
//     { isSigner: false, isWritable: false, pubkey: signerKey },
//     { isSigner: false, isWritable: true, pubkey: eventQueuePk },
//   ]
//
//
// }
function makeCancelOrderInstruction(programId, mangoGroupPk, ownerPk, marginAccountPk, dexProgramId, spotMarketPk, bidsPk, asksPk, openOrdersPk, signerKey, eventQueuePk, order) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: true, isWritable: false, pubkey: ownerPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        { isSigner: false, isWritable: false, pubkey: dexProgramId },
        { isSigner: false, isWritable: true, pubkey: spotMarketPk },
        { isSigner: false, isWritable: true, pubkey: bidsPk },
        { isSigner: false, isWritable: true, pubkey: asksPk },
        { isSigner: false, isWritable: true, pubkey: openOrdersPk },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: true, pubkey: eventQueuePk },
    ];
    const data = layout_1.encodeMangoInstruction({
        CancelOrder: {
            side: order.side,
            orderId: order.orderId,
        },
    });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeCancelOrderInstruction = makeCancelOrderInstruction;
function makeSettleFundsInstruction(programId, mangoGroupPk, ownerPk, marginAccountPk, dexProgramId, spotMarketPk, openOrdersPk, signerKey, spotMarketBaseVaultPk, spotMarketQuoteVaultPk, mangoBaseVaultPk, mangoQuoteVaultPk, dexSignerKey) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: true, isWritable: false, pubkey: ownerPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        { isSigner: false, isWritable: false, pubkey: dexProgramId },
        { isSigner: false, isWritable: true, pubkey: spotMarketPk },
        { isSigner: false, isWritable: true, pubkey: openOrdersPk },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: true, pubkey: spotMarketBaseVaultPk },
        { isSigner: false, isWritable: true, pubkey: spotMarketQuoteVaultPk },
        { isSigner: false, isWritable: true, pubkey: mangoBaseVaultPk },
        { isSigner: false, isWritable: true, pubkey: mangoQuoteVaultPk },
        { isSigner: false, isWritable: false, pubkey: dexSignerKey },
        { isSigner: false, isWritable: false, pubkey: token_instructions_1.TOKEN_PROGRAM_ID },
    ];
    const data = layout_1.encodeMangoInstruction({ SettleFunds: {} });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeSettleFundsInstruction = makeSettleFundsInstruction;
function makeSettleBorrowInstruction(programId, mangoGroupPk, marginAccountPk, walletPk, tokenIndex, nativeQuantity) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: true, isWritable: false, pubkey: walletPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
    ];
    const data = layout_1.encodeMangoInstruction({
        SettleBorrow: { tokenIndex: new bn_js_1.default(tokenIndex), quantity: nativeQuantity },
    });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeSettleBorrowInstruction = makeSettleBorrowInstruction;
//# sourceMappingURL=instruction.js.map