"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.encodeMangoInstruction = exports.MangoInstructionLayout = exports.selfTradeBehaviorLayout = exports.orderTypeLayout = exports.sideLayout = exports.MarginAccountLayout = exports.MangoGroupLayout = exports.MangoIndexLayout = exports.accountFlagsLayout = exports.WideBits = exports.U64F64 = exports.u128 = exports.u64 = exports.publicKeyLayout = exports.OPTIMAL_RATE = exports.OPTIMAL_UTIL = exports.MAX_RATE = exports.MANGO_GROUP_PADDING = exports.NUM_MARKETS = exports.NUM_TOKENS = void 0;
const buffer_layout_1 = require("buffer-layout");
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js"));
exports.NUM_TOKENS = 3;
exports.NUM_MARKETS = exports.NUM_TOKENS - 1;
exports.MANGO_GROUP_PADDING = 8 - (exports.NUM_TOKENS + exports.NUM_MARKETS) % 8;
exports.MAX_RATE = 1.0;
exports.OPTIMAL_UTIL = 0.7;
exports.OPTIMAL_RATE = 0.1;
class PublicKeyLayout extends buffer_layout_1.Blob {
    constructor(property) {
        super(32, property);
    }
    decode(b, offset) {
        return new web3_js_1.PublicKey(super.decode(b, offset));
    }
    encode(src, b, offset) {
        return super.encode(src.toBuffer(), b, offset);
    }
}
function publicKeyLayout(property = "") {
    return new PublicKeyLayout(property);
}
exports.publicKeyLayout = publicKeyLayout;
class BNLayout extends buffer_layout_1.Blob {
    constructor(number, property) {
        super(number, property);
        // restore prototype chain
        Object.setPrototypeOf(this, new.target.prototype);
    }
    decode(b, offset) {
        return new bn_js_1.default(super.decode(b, offset), 10, 'le');
    }
    encode(src, b, offset) {
        return super.encode(src.toArrayLike(Buffer, 'le', this['span']), b, offset);
    }
}
function u64(property = "") {
    return new BNLayout(8, property);
}
exports.u64 = u64;
function u128(property = "") {
    return new BNLayout(16, property);
}
exports.u128 = u128;
class U64F64Layout extends buffer_layout_1.Blob {
    constructor(property) {
        super(16, property);
    }
    decode(b, offset) {
        const raw = new bn_js_1.default(super.decode(b, offset), 10, 'le');
        // @ts-ignore
        return raw / Math.pow(2, 64);
    }
    encode(src, b, offset) {
        return super.encode(src.toArrayLike(Buffer, 'le', this['span']), b, offset);
    }
}
function U64F64(property = "") {
    return new U64F64Layout(property);
}
exports.U64F64 = U64F64;
class WideBits extends buffer_layout_1.Layout {
    constructor(property) {
        super(8, property);
        this._lower = buffer_layout_1.bits(buffer_layout_1.u32(), false);
        this._upper = buffer_layout_1.bits(buffer_layout_1.u32(), false);
    }
    addBoolean(property) {
        if (this._lower.fields.length < 32) {
            this._lower.addBoolean(property);
        }
        else {
            this._upper.addBoolean(property);
        }
    }
    decode(b, offset = 0) {
        const lowerDecoded = this._lower.decode(b, offset);
        const upperDecoded = this._upper.decode(b, offset + this._lower.span);
        return { ...lowerDecoded, ...upperDecoded };
    }
    replicate(property) {
        return super.replicate(property);
    }
    encode(src, b, offset = 0) {
        return (this._lower.encode(src, b, offset) +
            this._upper.encode(src, b, offset + this._lower.span));
    }
}
exports.WideBits = WideBits;
const ACCOUNT_FLAGS_LAYOUT = new WideBits(undefined);
ACCOUNT_FLAGS_LAYOUT.addBoolean('Initialized');
ACCOUNT_FLAGS_LAYOUT.addBoolean('MangoGroup');
ACCOUNT_FLAGS_LAYOUT.addBoolean('MarginAccount');
function accountFlagsLayout(property = 'accountFlags') {
    return ACCOUNT_FLAGS_LAYOUT.replicate(property); // TODO: when ts check is on, replicate throws error, doesn't compile
}
exports.accountFlagsLayout = accountFlagsLayout;
exports.MangoIndexLayout = buffer_layout_1.struct([
    u64('lastUpdate'),
    U64F64('borrow'),
    U64F64('deposit') // U64F64
]);
exports.MangoGroupLayout = buffer_layout_1.struct([
    accountFlagsLayout('accountFlags'),
    buffer_layout_1.seq(publicKeyLayout(), exports.NUM_TOKENS, 'tokens'),
    buffer_layout_1.seq(publicKeyLayout(), exports.NUM_TOKENS, 'vaults'),
    buffer_layout_1.seq(exports.MangoIndexLayout.replicate(), exports.NUM_TOKENS, 'indexes'),
    buffer_layout_1.seq(publicKeyLayout(), exports.NUM_MARKETS, 'spotMarkets'),
    buffer_layout_1.seq(publicKeyLayout(), exports.NUM_MARKETS, 'oracles'),
    u64('signerNonce'),
    publicKeyLayout('signerKey'),
    publicKeyLayout('dexProgramId'),
    buffer_layout_1.seq(U64F64(), exports.NUM_TOKENS, 'totalDeposits'),
    buffer_layout_1.seq(U64F64(), exports.NUM_TOKENS, 'totalBorrows'),
    U64F64('maintCollRatio'),
    U64F64('initCollRatio'),
    publicKeyLayout('srmVault'),
    publicKeyLayout('admin'),
    buffer_layout_1.seq(u64(), exports.NUM_TOKENS, 'borrowLimits'),
    buffer_layout_1.seq(buffer_layout_1.u8(), exports.NUM_TOKENS, 'mintDecimals'),
    buffer_layout_1.seq(buffer_layout_1.u8(), exports.NUM_MARKETS, 'oracleDecimals'),
    buffer_layout_1.seq(buffer_layout_1.u8(), exports.MANGO_GROUP_PADDING, 'padding')
]);
exports.MarginAccountLayout = buffer_layout_1.struct([
    accountFlagsLayout('accountFlags'),
    publicKeyLayout('mangoGroup'),
    publicKeyLayout('owner'),
    buffer_layout_1.seq(U64F64(), exports.NUM_TOKENS, 'deposits'),
    buffer_layout_1.seq(U64F64(), exports.NUM_TOKENS, 'borrows'),
    buffer_layout_1.seq(publicKeyLayout(), exports.NUM_MARKETS, 'openOrders'),
    u64('srmBalance')
]);
class EnumLayout extends buffer_layout_1.UInt {
    constructor(values, span, property) {
        super(span, property);
        this.values = values;
    }
    encode(src, b, offset) {
        if (this.values[src] !== undefined) {
            return super.encode(this.values[src], b, offset);
        }
        throw new Error('Invalid ' + this['property']);
    }
    decode(b, offset) {
        const decodedValue = super.decode(b, offset);
        const entry = Object.entries(this.values).find(([, value]) => value === decodedValue);
        if (entry) {
            return entry[0];
        }
        throw new Error('Invalid ' + this['property']);
    }
}
function sideLayout(property) {
    return new EnumLayout({ buy: 0, sell: 1 }, 4, property);
}
exports.sideLayout = sideLayout;
function orderTypeLayout(property) {
    return new EnumLayout({ limit: 0, ioc: 1, postOnly: 2 }, 4, property);
}
exports.orderTypeLayout = orderTypeLayout;
function selfTradeBehaviorLayout(property) {
    return new EnumLayout({ decrementTake: 0, cancelProvide: 1, abortTransaction: 2 }, 4, property);
}
exports.selfTradeBehaviorLayout = selfTradeBehaviorLayout;
exports.MangoInstructionLayout = buffer_layout_1.union(buffer_layout_1.u32('instruction'));
exports.MangoInstructionLayout.addVariant(0, buffer_layout_1.struct([]), 'InitMangoGroup'); // TODO this is unimplemented
exports.MangoInstructionLayout.addVariant(1, buffer_layout_1.struct([]), 'InitMarginAccount');
exports.MangoInstructionLayout.addVariant(2, buffer_layout_1.struct([u64('quantity')]), 'Deposit');
exports.MangoInstructionLayout.addVariant(3, buffer_layout_1.struct([u64('quantity')]), 'Withdraw');
exports.MangoInstructionLayout.addVariant(4, buffer_layout_1.struct([u64('tokenIndex'), u64('quantity')]), 'Borrow');
exports.MangoInstructionLayout.addVariant(5, buffer_layout_1.struct([u64('tokenIndex'), u64('quantity')]), 'SettleBorrow');
exports.MangoInstructionLayout.addVariant(6, buffer_layout_1.struct([buffer_layout_1.seq(u64(), exports.NUM_TOKENS, 'depositQuantities')]), 'Liquidate');
exports.MangoInstructionLayout.addVariant(7, buffer_layout_1.struct([u64('quantity')]), 'DepositSrm');
exports.MangoInstructionLayout.addVariant(8, buffer_layout_1.struct([u64('quantity')]), 'WithdrawSrm');
exports.MangoInstructionLayout.addVariant(9, buffer_layout_1.struct([
    sideLayout('side'),
    u64('limitPrice'),
    u64('maxBaseQuantity'),
    u64('maxQuoteQuantity'),
    selfTradeBehaviorLayout('selfTradeBehavior'),
    orderTypeLayout('orderType'),
    u64('clientId'),
    buffer_layout_1.u16('limit'),
]), 'PlaceOrder');
exports.MangoInstructionLayout.addVariant(10, buffer_layout_1.struct([]), 'SettleFunds');
exports.MangoInstructionLayout.addVariant(11, buffer_layout_1.struct([
    sideLayout('side'),
    u128('orderId')
]), 'CancelOrder');
exports.MangoInstructionLayout.addVariant(12, buffer_layout_1.struct([u64('clientId')]), 'CancelOrderByClientId');
exports.MangoInstructionLayout.addVariant(13, buffer_layout_1.struct([u64('tokenIndex'), u64('borrowLimit')]), 'ChangeBorrowLimit');
// @ts-ignore
const instructionMaxSpan = Math.max(...Object.values(exports.MangoInstructionLayout.registry).map((r) => r.span));
function encodeMangoInstruction(data) {
    const b = Buffer.alloc(instructionMaxSpan);
    const span = exports.MangoInstructionLayout.encode(data, b);
    return b.slice(0, span);
}
exports.encodeMangoInstruction = encodeMangoInstruction;
//# sourceMappingURL=layout.js.map