/// <reference types="node" />
import BN from "bn.js";
import { Connection, PublicKey } from '@solana/web3.js';
export declare abstract class Serialization {
    static loadWithConnection<T>(this: {
        new (data: any): T;
    }, key: PublicKey, connection: Connection): Promise<T>;
    static deserialize<T>(this: {
        new (data: any): T;
    }, data: Buffer): T;
    static serialize<T extends Serialization>(this: {
        new (data: any): T;
    }, data: object): Buffer;
    serialize(): Buffer;
    constructor(data: any);
}
declare class Submission {
    updatedAt: BN;
    value: BN;
    oracle: PublicKey;
    static schema: {
        kind: string;
        fields: (string | number[] | {
            encode: (key: PublicKey) => Buffer;
            decode: (buf: Uint8Array) => PublicKey;
        })[][];
    };
    constructor(data: any);
}
export interface IAggregatorConfig {
    decimals: number;
    description: string;
    restartDelay: number;
    rewardAmount: number;
    maxSubmissions: number;
    minSubmissions: number;
    rewardTokenAccount: PublicKey;
}
export declare class AggregatorConfig extends Serialization implements IAggregatorConfig {
    decimals: number;
    description: string;
    restartDelay: number;
    rewardAmount: number;
    maxSubmissions: number;
    minSubmissions: number;
    rewardTokenAccount: PublicKey;
    static schema: {
        kind: string;
        fields: ((string | number[] | {
            encode: (key: PublicKey) => Buffer;
            decode: (buf: Uint8Array) => PublicKey;
        })[] | (string | number[] | {
            encode: (str: String) => Buffer;
            decode: (bytes: Uint8Array) => string;
        })[])[];
    };
}
export declare class Submissions extends Serialization {
    isInitialized: boolean;
    submissions: Submission[];
    static size: number;
    static schema: {
        kind: string;
        fields: ((string | {
            encode: typeof boolToInt;
            decode: typeof intToBool;
        })[] | (string | (number | typeof Submission)[])[])[];
    };
    canSubmit(pk: PublicKey, cfg: AggregatorConfig): boolean;
    hadSubmitted(pk: PublicKey): boolean;
}
export declare class Round extends Serialization {
    id: BN;
    createdAt: BN;
    updatedAt: BN;
    static schema: {
        kind: string;
        fields: string[][];
    };
}
export declare class Answer extends Serialization {
    roundID: BN;
    median: BN;
    createdAt: BN;
    updatedAt: BN;
    static schema: {
        kind: string;
        fields: string[][];
    };
}
export declare class Aggregator extends Serialization {
    static size: number;
    config: AggregatorConfig;
    roundSubmissions: PublicKey;
    answerSubmissions: PublicKey;
    answer: Answer;
    round: Round;
    static schema: {
        kind: string;
        fields: ((string | number[] | {
            encode: (key: PublicKey) => Buffer;
            decode: (buf: Uint8Array) => PublicKey;
        })[] | (string | {
            encode: typeof boolToInt;
            decode: typeof intToBool;
        })[] | (string | typeof AggregatorConfig)[] | (string | typeof Round)[] | (string | typeof Answer)[])[];
    };
}
declare abstract class InstructionSerialization extends Serialization {
    serialize(): Buffer;
}
export declare class Initialize extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: (string | typeof AggregatorConfig)[][];
    };
}
export declare class Configure extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: (string | typeof AggregatorConfig)[][];
    };
}
export declare class AddOracle extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: (string | number[] | {
            encode: (str: String) => Buffer;
            decode: (bytes: Uint8Array) => string;
        })[][];
    };
}
export declare class RemoveOracle extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: never[];
    };
}
export declare class Withdraw extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: (string | string[])[][];
    };
}
export declare class Submit extends InstructionSerialization {
    static schema: {
        kind: string;
        fields: string[][];
    };
}
export declare class Instruction extends Serialization {
    enum: string;
    static schema: {
        kind: string;
        field: string;
        values: ((string | typeof Initialize)[] | (string | typeof AddOracle)[])[];
    };
    constructor(prop: {
        [key: string]: any;
    });
    get value(): any;
}
declare function intToBool(i: number): boolean;
declare function boolToInt(t: boolean): 1 | 0;
export declare class Oracle extends Serialization {
    static size: number;
    allowStartRound: BN;
    withdrawable: BN;
    static schema: {
        kind: string;
        fields: ((string | number[] | {
            encode: (key: PublicKey) => Buffer;
            decode: (buf: Uint8Array) => PublicKey;
        })[] | (string | number[] | {
            encode: (str: String) => Buffer;
            decode: (bytes: Uint8Array) => string;
        })[] | (string | {
            encode: typeof boolToInt;
            decode: typeof intToBool;
        })[])[];
    };
    canStartNewRound(round: BN): boolean;
}
export declare const schema: any;
export {};
//# sourceMappingURL=schema.d.ts.map