"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.schema = exports.Oracle = exports.Instruction = exports.Submit = exports.Withdraw = exports.RemoveOracle = exports.AddOracle = exports.Configure = exports.Initialize = exports.Aggregator = exports.Answer = exports.Round = exports.Submissions = exports.AggregatorConfig = exports.Serialization = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const borsh_1 = require("borsh");
const web3_js_1 = require("@solana/web3.js");
// const conn = new Connection("https://devnet.solana.com", 'singleGossip')
const MAX_ORACLES = 13;
const boolMapper = {
    encode: boolToInt,
    decode: intToBool,
};
const pubkeyMapper = {
    encode: (key) => {
        // if (key.constructor == PublicKey) {
        //   // key.
        // } else {
        //   key
        // }
        // TODO: support either account or public key
        return key.toBuffer();
    },
    decode: (buf) => {
        return new web3_js_1.PublicKey(buf);
    },
};
// support strings that can be contained in at most 32 bytes
const str32Mapper = {
    encode: (str) => {
        str = str.substr(0, 32).padEnd(32);
        return Buffer.from(str, "utf8").slice(0, 32); // truncate at 32 bytes
    },
    decode: (bytes) => {
        return Buffer.from(bytes).toString("utf8").trim();
    },
};
const u64Date = {
    encode: (date) => {
        return new bn_js_1.default(Math.floor(date.getTime() / 1000));
    },
    decode: (unixtime) => {
        return new Date(unixtime.toNumber() * 1000);
    },
};
class Serialization {
    static async loadWithConnection(key, connection) {
        const info = await connection.getAccountInfo(key);
        if (!info) {
            throw new Error("account does not exist");
        }
        return borsh_1.deserialize(exports.schema, this, info.data);
    }
    // public static async load<T>(
    //   this: { new (data: any): T },
    //   key: PublicKey
    // ): Promise<T> {
    //   const info = await conn.getAccountInfo(key, "recent")
    //   if (!info) {
    //     throw new Error("account does not exist")
    //   }
    //
    //   return deserialize(schema, this, info.data)
    // }
    static deserialize(data) {
        return borsh_1.deserialize(exports.schema, this, data);
    }
    static serialize(data) {
        return new this(data).serialize();
    }
    serialize() {
        let buf = Buffer.from(borsh_1.serialize(exports.schema, this));
        if (buf.length == 0) {
            throw new Error("serialized buffer is 0. something wrong with schema");
        }
        return buf;
    }
    // public toJSON(pretty = true) {
    //   return JSON.stringify(
    //     this[Serialization.DATA_KEY],
    //     jsonReplacer,
    //     pretty ? 2 : 0
    //   )
    // }
    // public static DATA_KEY = Symbol("DATA")
    constructor(data) {
        // this[Serialization.DATA_KEY] = data
        Object.assign(this, data);
    }
}
exports.Serialization = Serialization;
class Submission {
    constructor(data) {
        Object.assign(this, data);
    }
}
Submission.schema = {
    kind: "struct",
    fields: [
        ["updatedAt", "u64"],
        ["value", "u64"],
        ["oracle", [32], pubkeyMapper],
    ],
};
class AggregatorConfig extends Serialization {
}
exports.AggregatorConfig = AggregatorConfig;
AggregatorConfig.schema = {
    kind: "struct",
    fields: [
        ["description", [32], str32Mapper],
        ["decimals", "u8"],
        ["restartDelay", "u8"],
        ["maxSubmissions", "u8"],
        ["minSubmissions", "u8"],
        ["rewardAmount", "u64"],
        ["rewardTokenAccount", [32], pubkeyMapper],
    ],
};
class Submissions extends Serialization {
    // if not already submitted, and has empty spot
    canSubmit(pk, cfg) {
        if (this.hadSubmitted(pk)) {
            return false;
        }
        let emptyIndex = this.submissions.findIndex((s) => {
            return s.updatedAt.isZero();
        });
        return emptyIndex > 0 && emptyIndex < cfg.maxSubmissions;
    }
    hadSubmitted(pk) {
        return !!this.submissions.find((s) => {
            return s.oracle.equals(pk);
        });
    }
}
exports.Submissions = Submissions;
Submissions.size = 625;
Submissions.schema = {
    kind: "struct",
    fields: [
        ["isInitialized", "u8", boolMapper],
        ["submissions", [Submission, MAX_ORACLES]],
    ],
};
class Round extends Serialization {
}
exports.Round = Round;
Round.schema = {
    kind: "struct",
    fields: [
        ["id", "u64"],
        ["createdAt", "u64"],
        ["updatedAt", "u64"],
    ],
};
class Answer extends Serialization {
}
exports.Answer = Answer;
Answer.schema = {
    kind: "struct",
    fields: [
        ["roundID", "u64"],
        ["median", "u64"],
        ["createdAt", "u64"],
        ["updatedAt", "u64"],
    ],
};
class Aggregator extends Serialization {
}
exports.Aggregator = Aggregator;
Aggregator.size = 229;
Aggregator.schema = {
    kind: "struct",
    fields: [
        ["config", AggregatorConfig],
        ["isInitialized", "u8", boolMapper],
        ["owner", [32], pubkeyMapper],
        ["round", Round],
        ["roundSubmissions", [32], pubkeyMapper],
        ["answer", Answer],
        ["answerSubmissions", [32], pubkeyMapper],
    ],
};
class InstructionSerialization extends Serialization {
    serialize() {
        return new Instruction({ [this.constructor.name]: this }).serialize();
    }
}
class Initialize extends InstructionSerialization {
}
exports.Initialize = Initialize;
// public submitInterval!: number
// public minSubmissionValue!: number
// public maxSubmissionValue!: number
// public submissionDecimals!: number
// /// A short description of what is being reported
// public description!: string
Initialize.schema = {
    kind: "struct",
    fields: [["config", AggregatorConfig]],
};
class Configure extends InstructionSerialization {
}
exports.Configure = Configure;
Configure.schema = {
    kind: "struct",
    fields: [["config", AggregatorConfig]],
};
class AddOracle extends InstructionSerialization {
}
exports.AddOracle = AddOracle;
AddOracle.schema = {
    kind: "struct",
    fields: [["description", [32], str32Mapper]],
};
class RemoveOracle extends InstructionSerialization {
}
exports.RemoveOracle = RemoveOracle;
RemoveOracle.schema = {
    kind: "struct",
    fields: [],
};
class Withdraw extends InstructionSerialization {
}
exports.Withdraw = Withdraw;
Withdraw.schema = {
    kind: "struct",
    fields: [["faucetOwnerSeed", ["u8"]]],
};
class Submit extends InstructionSerialization {
}
exports.Submit = Submit;
Submit.schema = {
    kind: "struct",
    fields: [
        ["round_id", "u64"],
        ["value", "u64"],
    ],
};
class Instruction extends Serialization {
    constructor(prop) {
        super({});
        // deserializer calls the construction with `{ [enum]: value }`, so we need
        // to figure out the enum type
        //
        // expect only one key-value (what a retarded interface)
        for (let key of Object.keys(prop)) {
            this.enum = key;
            this[key] = prop[key];
            return;
        }
        throw new Error("not an expected enum object");
    }
    get value() {
        return this[this.enum];
    }
}
exports.Instruction = Instruction;
Instruction.schema = {
    kind: "enum",
    field: "enum",
    values: [
        [Initialize.name, Initialize],
        [Configure.name, Configure],
        [AddOracle.name, AddOracle],
        [RemoveOracle.name, RemoveOracle],
        [Submit.name, Submit],
    ],
};
function intToBool(i) {
    if (i == 0) {
        return false;
    }
    else {
        return true;
    }
}
function boolToInt(t) {
    if (t) {
        return 1;
    }
    else {
        return 0;
    }
}
class Oracle extends Serialization {
    canStartNewRound(round) {
        return this.allowStartRound.lte(round);
    }
}
exports.Oracle = Oracle;
Oracle.size = 113;
Oracle.schema = {
    kind: "struct",
    fields: [
        ["description", [32], str32Mapper],
        ["isInitialized", "u8", boolMapper],
        ["withdrawable", "u64"],
        ["allowStartRound", "u64"],
        ["aggregator", [32], pubkeyMapper],
        ["owner", [32], pubkeyMapper],
    ],
};
// if there is optional or variable length items, what is: borsh_utils::get_packed_len::<Submission>()?
//
// would panic given variable sized types
exports.schema = new Map([
    [Aggregator, Aggregator.schema],
    [Oracle, Oracle.schema],
    [Round, Round.schema],
    [Answer, Answer.schema],
    [AggregatorConfig, AggregatorConfig.schema],
    [Submissions, Submissions.schema],
    [Submission, Submission.schema],
    [Instruction, Instruction.schema],
    [Initialize, Initialize.schema],
    [AddOracle, AddOracle.schema],
    [Submit, Submit.schema],
]);
//# sourceMappingURL=schema.js.map