"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMultipleAccounts = exports.parseTokenAccountData = exports.ACCOUNT_LAYOUT = exports.getUnixTs = exports.promiseUndef = exports.getFilteredProgramAccounts = exports.nativeToUi = exports.uiToNative = exports.getMintDecimals = exports.createAccountInstruction = exports.zeroKey = void 0;
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js"));
const token_instructions_1 = require("@project-serum/serum/lib/token-instructions");
const buffer_layout_1 = require("buffer-layout");
exports.zeroKey = new web3_js_1.PublicKey(new Uint8Array(32));
async function createAccountInstruction(connection, payer, space, owner, lamports) {
    const account = new web3_js_1.Account();
    const instruction = web3_js_1.SystemProgram.createAccount({
        fromPubkey: payer,
        newAccountPubkey: account.publicKey,
        lamports: lamports ? lamports : await connection.getMinimumBalanceForRentExemption(space),
        space,
        programId: owner
    });
    return { account, instruction };
}
exports.createAccountInstruction = createAccountInstruction;
const MINT_LAYOUT = buffer_layout_1.struct([buffer_layout_1.blob(44), buffer_layout_1.u8('decimals'), buffer_layout_1.blob(37)]);
async function getMintDecimals(connection, mint) {
    if (mint.equals(token_instructions_1.WRAPPED_SOL_MINT)) {
        return 9;
    }
    const { data } = throwIfNull(await connection.getAccountInfo(mint), 'mint not found');
    const { decimals } = MINT_LAYOUT.decode(data);
    return decimals;
}
exports.getMintDecimals = getMintDecimals;
function throwIfNull(value, message = 'account not found') {
    if (value === null) {
        throw new Error(message);
    }
    return value;
}
function uiToNative(amount, decimals) {
    return new bn_js_1.default(Math.round(amount * Math.pow(10, decimals)));
}
exports.uiToNative = uiToNative;
function nativeToUi(amount, decimals) {
    return amount / Math.pow(10, decimals);
}
exports.nativeToUi = nativeToUi;
async function getFilteredProgramAccounts(connection, programId, filters) {
    // @ts-ignore
    const resp = await connection._rpcRequest('getProgramAccounts', [
        programId.toBase58(),
        {
            commitment: connection.commitment,
            filters,
            encoding: 'base64',
        },
    ]);
    if (resp.error) {
        throw new Error(resp.error.message);
    }
    return resp.result.map(({ pubkey, account: { data, executable, owner, lamports } }) => ({
        publicKey: new web3_js_1.PublicKey(pubkey),
        accountInfo: {
            data: Buffer.from(data[0], 'base64'),
            executable,
            owner: new web3_js_1.PublicKey(owner),
            lamports,
        },
    }));
}
exports.getFilteredProgramAccounts = getFilteredProgramAccounts;
async function promiseUndef() {
    return undefined;
}
exports.promiseUndef = promiseUndef;
const getUnixTs = () => {
    return new Date().getTime() / 1000;
};
exports.getUnixTs = getUnixTs;
exports.ACCOUNT_LAYOUT = buffer_layout_1.struct([
    buffer_layout_1.blob(32, 'mint'),
    buffer_layout_1.blob(32, 'owner'),
    buffer_layout_1.nu64('amount'),
    buffer_layout_1.blob(93)
]);
function parseTokenAccountData(data) {
    let { mint, owner, amount } = exports.ACCOUNT_LAYOUT.decode(data);
    return {
        mint: new web3_js_1.PublicKey(mint),
        owner: new web3_js_1.PublicKey(owner),
        amount,
    };
}
exports.parseTokenAccountData = parseTokenAccountData;
async function getMultipleAccounts(connection, publicKeys) {
    const publickKeyStrs = publicKeys.map((pk) => (pk.toBase58()));
    // @ts-ignore
    const resp = await connection._rpcRequest('getMultipleAccounts', [publickKeyStrs]);
    if (resp.error) {
        throw new Error(resp.error.message);
    }
    return resp.result.value.map(({ data, executable, lamports, owner }, i) => ({
        publicKey: publicKeys[i],
        accountInfo: {
            data: Buffer.from(data[0], 'base64'),
            executable,
            owner: new web3_js_1.PublicKey(owner),
            lamports,
        },
    }));
}
exports.getMultipleAccounts = getMultipleAccounts;
//# sourceMappingURL=utils.js.map